package com.ruoyi.ss.bonus.service.impl;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.ruoyi.common.domain.vo.LocalDateDecimalVO;
import com.ruoyi.common.domain.vo.LongDecimalVO;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.BonusQuery;
import com.ruoyi.ss.bonus.domain.BonusVO;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusBstType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.domain.vo.BonusDailyAmountVO;
import com.ruoyi.ss.bonus.domain.vo.BonusMonthAmountVO;
import com.ruoyi.ss.bonus.mapper.BonusMapper;
import com.ruoyi.ss.bonus.service.BonusService;
import com.ruoyi.ss.recordBalance.domain.enums.RecordBalanceBstType;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.UserAssembler;
import com.ruoyi.ss.user.service.UserService;
import com.ruoyi.system.service.ISysConfigService;

import lombok.extern.slf4j.Slf4j;

/**
 * 分成明细Service业务层处理
 *
 * @author ruoyi
 * @date 2024-08-20
 */
@Service
@Slf4j
public class BonusServiceImpl implements BonusService
{
    @Autowired
    private BonusMapper bonusMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private UserService userService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private UserAssembler userAssembler;

    /**
     * 查询分成明细
     *
     * @param id 分成明细主键
     * @return 分成明细
     */
    @Override
    public BonusVO selectBonusById(Long id)
    {
        return bonusMapper.selectBonusById(id);
    }

    /**
     * 查询分成明细列表
     *
     * @param bonus 分成明细
     * @return 分成明细
     */
    @Override
    public List<BonusVO> selectBonusList(BonusQuery bonus)
    {
        return bonusMapper.selectBonusList(bonus);
    }

    /**
     * 新增分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int insertBonus(Bonus bonus)
    {
        this.buildBeforeCreate(bonus);
        return bonusMapper.insertBonus(bonus);
    }

    // 补全新增前的数据
    private void buildBeforeCreate(Bonus bonus) {
        bonus.setCreateTime(DateUtils.getNowDate());
    }

    /**
     * 修改分成明细
     *
     * @param bonus 分成明细
     * @return 结果
     */
    @Override
    public int updateBonus(Bonus bonus)
    {
        return bonusMapper.updateBonus(bonus);
    }

    /**
     * 批量删除分成明细
     *
     * @param ids 需要删除的分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusByIds(Long[] ids)
    {
        return bonusMapper.deleteBonusByIds(ids);
    }

    /**
     * 删除分成明细信息
     *
     * @param id 分成明细主键
     * @return 结果
     */
    @Override
    public int deleteBonusById(Long id)
    {
        return bonusMapper.deleteBonusById(id);
    }

    @Override
    public int batchInsert(List<Bonus> bonusList) {
        if (CollectionUtils.isEmptyElement(bonusList)) {
            return 0;
        }
        for (Bonus bonus : bonusList) {
            this.buildBeforeCreate(bonus);
        }
        return bonusMapper.batchInsert(bonusList);
    }

    @Override
    public int payBonus(BonusVO bonus) {
        if (bonus == null) {
            return 0;
        }
        Integer result = transactionTemplate.execute(status -> {
            // 更新分成状态为已分成
            int pay = bonusMapper.pay(bonus.getId(), bonus.getWaitAmount(), LocalDateTime.now());
            ServiceUtil.assertion(pay != 1, "分成打款失败，待分金额不足或状态已改变");

            // 若需要打款到余额，根据类型，添加金额
            if (bonus.getToBalance() != null && bonus.getToBalance()) {
                if (BonusArrivalType.userList().contains(bonus.getArrivalType())) {
                    String reason = this.getReason(bonus.getBstType(), bonus.getBillNo());
                    RecordBalanceBstType bstType = this.getBstType(bonus.getBstType());
                    int add = userService.addBalance(bonus.getArrivalId(), bonus.getWaitAmount(), reason, bstType, bonus.getBillId());
                    ServiceUtil.assertion(add != 1, "增加账户金额失败");
                } else if (BonusArrivalType.deptList().contains(bonus.getArrivalType())) {
//                    add = deptService.addBalance(bonus.getArrivalId(), bonus.getAmount(), String.format("订单分成:%s", bonus.getBillNo()), RecordBalanceBstType.RECHARGE, bonus.getBillId());
                }
            }

            return pay;
        });

        return result == null ? 0 : result;
    }

    private RecordBalanceBstType getBstType(String bstType) {
        if (BonusBstType.RECHARGE.getType().equals(bstType)) {
            return RecordBalanceBstType.RECHARGE;
        } else if (BonusBstType.VIP.getType().equals(bstType)) {
            return RecordBalanceBstType.VIP;
        }
        return null;
    }

    // 获取打款理由
    private String getReason(String bstType, String billNo) {
        if (BonusBstType.RECHARGE.getType().equals(bstType)) {
            return String.format("充值订单分成:%s", billNo);
        } else if (BonusBstType.VIP.getType().equals(bstType)) {
            return String.format("VIP订单分成:%s", billNo);
        }
        return null;
    }

    @Override
    public int prePay(List<? extends Bonus> bonusList) {
        if (CollectionUtils.isEmptyElement(bonusList)) {
            return 0;
        }

        // 设置预计分成时间等数据
        LocalDateTime now = LocalDateTime.now();

        // 获取用户列表
        List<SmUserVO> userList = userService.selectByUserIds(bonusList.stream()
                .filter(item -> BonusArrivalType.userList().contains(item.getArrivalType()))
                .map(Bonus::getArrivalId)
                .collect(Collectors.toList())
        );
        // 拼接用户实际到账延迟
        userAssembler.assembleRealArrivalDelay(userList);

        for (Bonus bonus : bonusList) {
            // 设置预计分成时间
            if (BonusArrivalType.userList().contains(bonus.getArrivalType())) {
                SmUserVO user = userList.stream().filter(item -> item.getUserId().equals(bonus.getArrivalId())).findFirst().orElse(null);
                if (user == null) {
                    continue;
                }
                bonus.setPrePayTime(now.plusHours(user.getRealArrivalDelay()));
            } else {
                bonus.setPrePayTime(now);
            }
        }

        // 更新数据库
        Integer result = transactionTemplate.execute(status -> {
            int rows = bonusMapper.prePay(bonusList);
            ServiceUtil.assertion(rows != bonusList.size(), "预分成失败，请刷新后重试");
            return rows;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int refundWhenWaitDivide(Long id, BigDecimal amount) {
        if (id == null) {
            return 0;
        }
        ServiceUtil.assertion(amount.compareTo(BigDecimal.ZERO) < 0, "退款金额必须大于0");
        return bonusMapper.refundWhenWaitDivide(id, amount);
    }

    @Override
    public int refundWhenDividend(Long id, BigDecimal amount) {
        if (id == null) {
            return 0;
        }
        ServiceUtil.assertion(amount.compareTo(BigDecimal.ZERO) < 0, "退款金额必须大于0");
        return bonusMapper.refundWhenDividend(id, amount);
    }

    @Override
    public List<BonusMonthAmountVO> selectMonthPayedAmount(BonusQuery query) {
        return bonusMapper.selectMonthPayedAmount(query);
    }

    @Override
    public List<BonusDailyAmountVO> selectDailyAmount(BonusQuery query) {
        return bonusMapper.selectDailyAmount(query);
    }

    @Override
    public int selectCount(BonusQuery query) {
        return bonusMapper.selectCount(query);
    }

    @Override
    public int payBonusBeforeTime(LocalDateTime time) {
        // 查询预计分成时间当前时间之前，且为待分成的分成单
        BonusQuery query = new BonusQuery();
        query.setStatus(BonusStatus.WAIT_DIVIDE.getStatus());
        query.setPrePayTimeEnd(time);
        List<BonusVO> list = this.selectBonusList(query);

        if (CollectionUtils.isEmptyElement(list)) {
            log.info("暂无待分成的分成单");
            return 0;
        }

        int sum = 0;
        for (BonusVO bonus : list) {
            try {
                sum += this.payBonus(bonus);
            } catch (Exception e) {
                log.warn("分成打款失败：id={}, e={}", bonus.getId(), e.getMessage());
            }
        }

        return sum;
    }

    @Override
    public List<LongDecimalVO> selectWaitAmountGroupByArrival(BonusQuery query) {
        return bonusMapper.selectWaitAmountGroupByArrival(query);
    }

    @Override
    public List<LongDecimalVO> selectPayedAmountGroupByArrival(BonusQuery query) {
        return bonusMapper.selectPayedAmountGroupByArrival(query);
    }

    @Override
    public int payBonusById(Long id) {
        BonusVO bonus = this.selectBonusById(id);
        return this.payBonus(bonus);
    }

    @Override
    public List<LocalDateDecimalVO> selectDailyPayedAmount(BonusQuery query) {
        return bonusMapper.selectDailyPayedAmount(query);
    }

    @Override
    public BigDecimal selectSumOfPayedAmount(BonusQuery query) {
        return bonusMapper.selectSumOfPayedAmount(query);
    }


    /**
     * FIXME
     * !!!! 重要 !!!!
     * 临时用法，2025-01-17 发生大规模的退款账变BUG，需要临时处理
     * 这个方法是用来直接修改数据库，所以需要谨慎使用
     */
    @Override
    public int subtractBalance(Long id, BigDecimal amount) {
        BonusVO bonus = this.selectBonusById(id);
        ServiceUtil.assertion(bonus == null, "分成不存在");
        ServiceUtil.assertion(bonus.getToBalance() == null || !bonus.getToBalance(), "分成未变动余额");
        ServiceUtil.assertion(!BonusArrivalType.userList().contains(bonus.getArrivalType()), "该分成方不是用户");
        ServiceUtil.assertion(bonus.getByHand(), "该分成已经手动处理过了");

        Integer result = transactionTemplate.execute(status -> {
            Bonus data = new BonusVO();
            data.setId(id);
            data.setByHand(true);
            int update = this.updateBonus(data);
            ServiceUtil.assertion(update != 1, "修改状态失败，请刷新后重试");

            userService.subtractBalance(bonus.getArrivalId(), amount, false, String.format("订单退款:%s", bonus.getBillNo()), RecordBalanceBstType.RECHARGE, bonus.getBillId());

            return update;
        });
        return result == null ? 0 : result;
    }

    @Override
    public List<BonusVO> selectBonusByBstId(BonusBstType bstType, Long bstId) {
        if (bstType == null || bstId == null) {
            return Collections.emptyList();
        }
        BonusQuery query = new BonusQuery();
        query.setBstType(bstType.getType());
        query.setBillId(bstId);
        return this.selectBonusList(query);
    }

    private int batchUpdateAmount(List<? extends Bonus> list) {
        if (CollectionUtils.isEmptyElement(list)) {
            return 0;
        }
        return bonusMapper.batchUpdateAmount(list);
    }

    @Override
    public BigDecimal selectSumOfAmount(BonusQuery query) {
        return bonusMapper.selectSumOfAmount(query);
    }

    @Override
    public BigDecimal selectSumOfWaitAmount(BonusQuery query) {
        return bonusMapper.selectSumOfWaitAmount(query);
    }

    @Override
    public int selectBillCount(BonusQuery query) {
        return bonusMapper.selectBillCount(query);
    }
}
