package com.ruoyi.ss.bonus.utils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.List;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.BonusVO;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.user.domain.SmUserVO;

/**
 * @author wjh
 * 2025/1/21
 */
public class BonusUtil {

    /**
     * 平台全收服务费分成处理
     */
    public static void partBonusAllPlatform(Bonus platform, BigDecimal money) {
        setAmount(platform, money);
    }

    /**
     * 最低服务费分成处理
     */
    public static void partBonusMinService(List<? extends Bonus> bonusList, BigDecimal money, Bonus platform, BigDecimal minService) {
        // 平台设置分成为最低金额
        setAmount(platform, minService);
        BigDecimal decimal100 = new BigDecimal("100");
        // 计算其他分成方分成金额
        BigDecimal remain = money.subtract(platform.getAmount());           // 剩余可分成金额
        BigDecimal remainPoint = decimal100.subtract(platform.getPoint());  // 剩余总百分比
        for (Bonus bonus : bonusList) {
            if (bonus.getArrivalType().equals(BonusArrivalType.PLATFORM.getType())) {
                continue;
            }
            // 计算分成比例占剩余可分成金额的比例
            BigDecimal bonusPoint = bonus.getPoint().multiply(decimal100).divide(remainPoint, 2, RoundingMode.HALF_UP);
            BigDecimal amount = remain.multiply(bonusPoint).divide(decimal100, 2, RoundingMode.HALF_UP);
            setAmount(bonus, amount);
        }
    }

    /**
     * 基础处理分成
     */
    public static void partBonusNormal(List<? extends Bonus> bonusList, BigDecimal money) {
        BigDecimal decimal100 = new BigDecimal("100");
        // 循环遍历，构造分成金额
        for (Bonus bonus : bonusList) {
            BigDecimal amount = money.multiply(bonus.getPoint()).divide(decimal100, 2, RoundingMode.HALF_UP);
            setAmount(bonus, amount);
        }
    }


    private static void setAmount(Bonus bonus, BigDecimal amount) {
        ServiceUtil.assertion(amount.compareTo(BigDecimal.ZERO) < 0, "分成金额不允许小于0");
        bonus.setAmount(amount);
    }

    /**
     * 处理分成的误差
     */
    public static void handlePartDiff(List<? extends Bonus> bonusList, BigDecimal diff) {
        // 若误差金额为正数或者0，则交给第一个分成方处理
        if (diff.compareTo(BigDecimal.ZERO) >= 0) {
            Bonus bonus = bonusList.get(0);
            setAmount(bonus, bonus.getAmount().add(diff));
        }
        // 否则，若为负数，则多个分成方处理
        else {
            // 多个分成方共同承担误差
            BigDecimal remainingDiff = diff;
            for (Bonus bonus : bonusList) {
                if (bonus == null || bonus.getAmount().compareTo(BigDecimal.ZERO) <= 0) {
                    continue;
                }

                // 计算当前分成方可以承担的误差金额（确保不会导致金额为负）
                BigDecimal maxDeductible = bonus.getAmount();  // 最大可扣除金额
                BigDecimal adjustAmount = maxDeductible.min(remainingDiff.abs());

                // 直接调整金额
                setAmount(bonus, bonus.getAmount().subtract(adjustAmount));
                remainingDiff = remainingDiff.add(adjustAmount);

                // 如果误差已经分配完毕，则退出
                if (remainingDiff.compareTo(BigDecimal.ZERO) == 0) {
                    return;
                }
            }

            // 如果所有分成方都处理完了还有剩余误差，抛出异常
            if (remainingDiff.compareTo(BigDecimal.ZERO) != 0) {
                throw new ServiceException("分成金额误差处理失败：无法完全分配误差金额，剩余误差：" + remainingDiff);
            }
        }
    }


    /**
     * 处理退款金额分配的误差
     * @param refundList 退款列表
     * @param bonusList 原始分成列表
     * @param dividedAmount 已分配金额
     * @param refundAmount 需要退款总金额
     */
    public static void handleRefundAmountDiff(List<Bonus> refundList, List<BonusVO> bonusList, BigDecimal dividedAmount, BigDecimal refundAmount) {
        if (dividedAmount.compareTo(refundAmount) == 0) {
            return;
        }
        // 多个分成方共同承担误差
        BigDecimal remainingDiff = refundAmount.subtract(dividedAmount);
        for (Bonus refundBonus : refundList) {
            // 获取原始分成记录
            BonusVO originalBonus = bonusList.stream()
                    .filter(b -> b.getId().equals(refundBonus.getId()))
                    .findFirst()
                    .orElse(null);

            if (originalBonus == null) {
                continue;
            }

            if (remainingDiff.compareTo(BigDecimal.ZERO) > 0) {
                // 处理正数误差（少退了）：需要增加退款金额
                // 计算当前分成方的剩余可退金额
                BigDecimal availableRefund = originalBonus.getAmount()
                        .subtract(originalBonus.getRefundAmount() == null ? BigDecimal.ZERO : originalBonus.getRefundAmount())
                        .subtract(refundBonus.getRefundAmount());

                // 如果剩余可退金额足够承担误差
                if (availableRefund.compareTo(BigDecimal.ZERO) > 0) {
                    // 取可用退款金额与剩余误差中的较小值
                    BigDecimal adjustAmount = availableRefund.min(remainingDiff);
                    refundBonus.setRefundAmount(refundBonus.getRefundAmount().add(adjustAmount));
                    remainingDiff = remainingDiff.subtract(adjustAmount);
                }
            } else {
                // 处理负数误差（多退了）：需要减少退款金额
                // 计算当前退款金额
                BigDecimal currentRefund = refundBonus.getRefundAmount();
                if (currentRefund.compareTo(BigDecimal.ZERO) > 0) {
                    // 取当前退款金额与剩余需要减少金额的较小值
                    BigDecimal adjustAmount = currentRefund.min(remainingDiff.abs());
                    refundBonus.setRefundAmount(currentRefund.subtract(adjustAmount));
                    remainingDiff = remainingDiff.add(adjustAmount);
                }
            }

            // 如果误差已经分配完毕，则退出
            if (remainingDiff.compareTo(BigDecimal.ZERO) == 0) {
                return;
            }
        }

        // 如果仍然无法完全分配误差,抛出异常
        if (remainingDiff.compareTo(BigDecimal.ZERO) != 0) {
            throw new ServiceException("退款金额分配出错:无法处理误差金额，剩余误差：" + remainingDiff);
        }
    }

    public static void partBonusAmount(List<? extends Bonus> bonusList, BigDecimal money, BigDecimal minService) {
        BigDecimal decimal100 = new BigDecimal(100);

        // 获取平台的分成
        Bonus platform = bonusList.stream().filter(bonus -> bonus.getArrivalType().equals(BonusArrivalType.PLATFORM.getType())).findFirst().orElse(null);
        // 平台存在分成的情况，需要处理最低分成
        if (platform != null) {
            BigDecimal platformAmount = money.multiply(platform.getPoint()).divide(decimal100, 2, RoundingMode.HALF_UP);    // 平台预计分成金额
            // 若总金额 < 最低分成，则平台全收
            if (money.compareTo(minService) < 0) {
                BonusUtil.partBonusAllPlatform(platform, money);
            }
            // 若平台分成 < 最低金额，则收取最低金额，其他金额给其他分成方分成
            else if (platformAmount.compareTo(minService) < 0) {
                BonusUtil.partBonusMinService(bonusList, money, platform, minService);
            }
            // 其余正常收取
            else {
                BonusUtil.partBonusNormal(bonusList, money);
            }
        }
        // 其余情况，正常收取费用
        else {
            BonusUtil.partBonusNormal(bonusList, money);
        }

        // 误差处理，将误差值交给可以处理的分成方处理
        BigDecimal dividedAmount = CollectionUtils.sumDecimal(bonusList, Bonus::getAmount);
        BonusUtil.handlePartDiff(bonusList, money.subtract(dividedAmount));
        // 处理误差后的分配金额
        dividedAmount = CollectionUtils.sumDecimal(bonusList, Bonus::getAmount);
        ServiceUtil.assertion(dividedAmount.compareTo(money) != 0, "分成金额分配出错");

        for (Bonus bonus : bonusList) {
            bonus.setWaitAmount(bonus.getAmount());
        }
    }
}
